CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
92 lines
2.9 KiB
C++
92 lines
2.9 KiB
C++
#include <algorithm> // for std::min
|
|
#include <hip/hip_runtime_api.h> // for hip functions
|
|
#include <rocsolver/rocsolver.h> // for all the rocsolver C interfaces and type declarations
|
|
#include <stdio.h> // for size_t, printf
|
|
#include <vector>
|
|
|
|
// Example: Compute the QR Factorization of a matrix on the GPU
|
|
|
|
void get_example_matrix(std::vector<double>& hA,
|
|
rocblas_int& M,
|
|
rocblas_int& N,
|
|
rocblas_int& lda) {
|
|
// a *very* small example input; not a very efficient use of the API
|
|
const double A[3][3] = { { 12, -51, 4},
|
|
{ 6, 167, -68},
|
|
{ -4, 24, -41} };
|
|
M = 3;
|
|
N = 3;
|
|
lda = 3;
|
|
// note: rocsolver matrices must be stored in column major format,
|
|
// i.e. entry (i,j) should be accessed by hA[i + j*lda]
|
|
hA.resize(size_t(lda) * N);
|
|
for (size_t i = 0; i < M; ++i) {
|
|
for (size_t j = 0; j < N; ++j) {
|
|
// copy A (2D array) into hA (1D array, column-major)
|
|
hA[i + j*lda] = A[i][j];
|
|
}
|
|
}
|
|
}
|
|
|
|
// We use rocsolver_dgeqrf to factor a real M-by-N matrix, A.
|
|
// See https://rocm.docs.amd.com/projects/rocSOLVER/en/latest/api/lapack.html#rocsolver-type-geqrf
|
|
int main() {
|
|
rocblas_int M; // rows
|
|
rocblas_int N; // cols
|
|
rocblas_int lda; // leading dimension
|
|
std::vector<double> hA; // input matrix on CPU
|
|
get_example_matrix(hA, M, N, lda);
|
|
|
|
// let's print the input matrix, just to see it
|
|
printf("A = [\n");
|
|
for (size_t i = 0; i < M; ++i) {
|
|
printf(" ");
|
|
for (size_t j = 0; j < N; ++j) {
|
|
printf("% .3f ", hA[i + j*lda]);
|
|
}
|
|
printf(";\n");
|
|
}
|
|
printf("]\n");
|
|
|
|
// initialization
|
|
rocblas_handle handle;
|
|
rocblas_create_handle(&handle);
|
|
|
|
// calculate the sizes of our arrays
|
|
size_t size_A = size_t(lda) * N; // count of elements in matrix A
|
|
size_t size_piv = size_t(std::min(M, N)); // count of Householder scalars
|
|
|
|
// allocate memory on GPU
|
|
double *dA, *dIpiv;
|
|
hipMalloc(&dA, sizeof(double)*size_A);
|
|
hipMalloc(&dIpiv, sizeof(double)*size_piv);
|
|
|
|
// copy data to GPU
|
|
hipMemcpy(dA, hA.data(), sizeof(double)*size_A, hipMemcpyHostToDevice);
|
|
|
|
// compute the QR factorization on the GPU
|
|
rocsolver_dgeqrf(handle, M, N, dA, lda, dIpiv);
|
|
|
|
// copy the results back to CPU
|
|
std::vector<double> hIpiv(size_piv); // array for householder scalars on CPU
|
|
hipMemcpy(hA.data(), dA, sizeof(double)*size_A, hipMemcpyDeviceToHost);
|
|
hipMemcpy(hIpiv.data(), dIpiv, sizeof(double)*size_piv, hipMemcpyDeviceToHost);
|
|
|
|
// the results are now in hA and hIpiv
|
|
// we can print some of the results if we want to see them
|
|
printf("R = [\n");
|
|
for (size_t i = 0; i < M; ++i) {
|
|
printf(" ");
|
|
for (size_t j = 0; j < N; ++j) {
|
|
printf("% .3f ", (i <= j) ? hA[i + j*lda] : 0);
|
|
}
|
|
printf(";\n");
|
|
}
|
|
printf("]\n");
|
|
|
|
// clean up
|
|
hipFree(dA);
|
|
hipFree(dIpiv);
|
|
rocblas_destroy_handle(handle);
|
|
}
|